@@ -11,6 +11,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import absolute_import, division, print_function
+
 INCLUDES = """
 #ifndef OPENSSL_NO_EC
 #include <openssl/ec.h>
@@ -23,14 +25,10 @@ TYPES = """
 static const int Cryptography_HAS_EC;
 
 typedef ... EC_KEY;
-
-static const int NID_X9_62_prime192v1;
-static const int NID_X9_62_prime192v2;
-static const int NID_X9_62_prime192v3;
-static const int NID_X9_62_prime239v1;
-static const int NID_X9_62_prime239v2;
-static const int NID_X9_62_prime239v3;
-static const int NID_X9_62_prime256v1;
+typedef struct {
+    int nid;
+    const char *comment;
+} EC_builtin_curve;
 """
 
 FUNCTIONS = """
@@ -39,14 +37,22 @@ FUNCTIONS = """
 MACROS = """
 EC_KEY *EC_KEY_new_by_curve_name(int);
 void EC_KEY_free(EC_KEY *);
+
+size_t EC_get_builtin_curves(EC_builtin_curve *, size_t);
+
 """
 
 CUSTOMIZATIONS = """
 #ifdef OPENSSL_NO_EC
 static const long Cryptography_HAS_EC = 0;
 typedef void EC_KEY;
+typedef struct {
+    int nid;
+    const char *comment;
+} EC_builtin_curve;
 EC_KEY* (*EC_KEY_new_by_curve_name)(int) = NULL;
 void (*EC_KEY_free)(EC_KEY *) = NULL;
+size_t (*EC_get_builtin_curves)(EC_builtin_curve *, size_t) = NULL;
 #else
 static const long Cryptography_HAS_EC = 1;
 #endif
@@ -56,5 +62,6 @@ CONDITIONAL_NAMES = {
     "Cryptography_HAS_EC": [
         "EC_KEY_new_by_curve_name",
         "EC_KEY_free",
+        "EC_get_builtin_curves",
     ],
 }
