@@ -19,7 +19,7 @@ import pytest
 
 from cryptography import utils
 from cryptography.exceptions import (
-    UnsupportedAlgorithm, AlreadyFinalized,
+    AlreadyFinalized, _Reasons
 )
 from cryptography.hazmat.primitives import interfaces
 from cryptography.hazmat.primitives.ciphers import (
@@ -29,6 +29,7 @@ from cryptography.hazmat.primitives.ciphers import (
 from .utils import (
     generate_aead_exception_test, generate_aead_tag_exception_test
 )
+from ...utils import raises_unsupported_algorithm
 
 
 @utils.register_interface(interfaces.Mode)
@@ -116,10 +117,10 @@ class TestCipherContext(object):
         cipher = Cipher(
             DummyCipher(), mode, backend
         )
-        with pytest.raises(UnsupportedAlgorithm):
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_CIPHER):
             cipher.encryptor()
 
-        with pytest.raises(UnsupportedAlgorithm):
+        with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_CIPHER):
             cipher.decryptor()
 
     def test_incorrectly_padded(self, backend):
